# Generated by Django 3.1.1 on 2020-09-27 17:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("timezones", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="TimezoneData",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "timezone_name",
                    models.CharField(
                        help_text="Name of the timezone", max_length=255, unique=True
                    ),
                ),
                (
                    "utc_offset",
                    models.CharField(help_text="UTC of the timezone", max_length=255),
                ),
                (
                    "panel_id",
                    models.CharField(
                        help_text="ID of the timezone panel in frontend",
                        max_length=255,
                        unique=True,
                    ),
                ),
            ],
            options={
                "verbose_name": "Timezone Data",
                "verbose_name_plural": "Timezone Data",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Timezones",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "panel_name",
                    models.CharField(
                        help_text="Name of the timezone panel",
                        max_length=255,
                        unique=True,
                    ),
                ),
                (
                    "is_enabled",
                    models.BooleanField(
                        default=True,
                        help_text="Whether this timezone is enabled or not",
                    ),
                ),
                (
                    "timezone",
                    models.ForeignKey(
                        help_text="Selected timezone",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="timezones.timezonedata",
                    ),
                ),
            ],
            options={
                "verbose_name": "Timezone",
                "verbose_name_plural": "Timezones",
                "default_permissions": (),
            },
        ),
    ]



# -*- coding: utf-8 -*-

"""
app config
"""

from django.apps import AppConfig

from timezones import __version__


class AaTimezonesConfig(AppConfig):
    """
    AaTimezonesConfig
    """

    name: str = "timezones"
    label: str = "timezones"
    verbose_name: str = "Timezones v{}".format(__version__)



# -*- coding: utf-8 -*-

"""
page urls config
"""

from django.conf.urls import url

from timezones import views


app_name: str = "timezones"

urlpatterns = [
    url(r"^$", views.index, name="index"),
    url(r"^(?P<timestamp>[0-9]+)/$", views.index, name="index"),
]



# -*- coding: utf-8 -*-

"""
admin backend config
"""

from django.contrib import admin

# Register your models here.
from timezones.models import Timezones


@admin.register(Timezones)
class TimezonesAdmin(admin.ModelAdmin):
    """
    timezones admin
    configure timezones that should be displayed
    """

    list_display = ("_panel_name", "_timezone", "is_enabled")
    ordering = ("panel_name",)
    list_filter = ("is_enabled",)

    @classmethod
    def _panel_name(cls, obj):
        return obj.panel_name

    _panel_name.short_description = "Panel Name"
    _panel_name.admin_order_field = "panel_name"

    @classmethod
    def _timezone(cls, obj):
        return obj.timezone.timezone_name

    _timezone.short_description = "Timezone"
    _timezone.admin_order_field = "timezone__timezone_name"

    actions = (
        "mark_as_active",
        "mark_as_inactive",
    )

    def mark_as_active(self, request, queryset):
        """
        Mark fleet type as active
        :param request:
        :param queryset:
        """

        notifications_count = 0

        for obj in queryset:
            obj.is_enabled = True
            obj.save()

            notifications_count += 1

        self.message_user(
            request, "{} timezone(s) activated".format(notifications_count)
        )

    mark_as_active.short_description = "Activate selected timezone(s)"

    def mark_as_inactive(self, request, queryset):
        """
        Mark fleet type as inactive
        :param request:
        :param queryset:
        """

        notifications_count = 0

        for obj in queryset:
            obj.is_enabled = False
            obj.save()

            notifications_count += 1

        self.message_user(
            request, "{} timezone(s) deactivated".format(notifications_count)
        )

    mark_as_inactive.short_description = "Deactivate selected timezone(s)"



# -*- coding: utf-8 -*-

"""
the views
"""
from typing import Dict, List, Union

from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import render
from timezones.models import Timezones

from timezones import __title__


@login_required
@permission_required("timezones.basic_access")
def index(request, timestamp: int = None):
    """
    index view

    :param request:
    :param timestamp:
    :return:
    """

    AA_TIMEZONE_DEFAULT_PANELS: List[Dict[str, Union[str, Dict[str, str]]]] = [
        {
            "panel_name": "US / Pacific",
            "timezone": {
                "timezone_name": "US/Pacific",
                "panel_id": "us-pacific",
            },
        },
        {
            "panel_name": "US / Mountain",
            "timezone": {
                "timezone_name": "US/Mountain",
                "panel_id": "us-mountain",
            },
        },
        {
            "panel_name": "US / Central",
            "timezone": {
                "timezone_name": "US/Central",
                "panel_id": "us-central",
            },
        },
        {
            "panel_name": "US / Eastern",
            "timezone": {
                "timezone_name": "US/Eastern",
                "panel_id": "us-eastern",
            },
        },
        {
            "panel_name": "EU / Western",
            "timezone": {
                "timezone_name": "Europe/London",
                "panel_id": "eu-western",
            },
        },
        {
            "panel_name": "EU / Central",
            "timezone": {
                "timezone_name": "Europe/Berlin",
                "panel_id": "eu-central",
            },
        },
        {
            "panel_name": "EU / Eastern",
            "timezone": {
                "timezone_name": "Europe/Istanbul",
                "panel_id": "eu-eastern",
            },
        },
        {
            "panel_name": "Russia / Moscow",
            "timezone": {
                "timezone_name": "Europe/Moscow",
                "panel_id": "russia-moscow",
            },
        },
        {
            "panel_name": "Russia / Siberia & China / Shanghai",
            "timezone": {
                "timezone_name": "Asia/Shanghai",
                "panel_id": "asia-shanghai",
            },
        },
        {
            "panel_name": "Australia / Sydney",
            "timezone": {
                "timezone_name": "Australia/ACT",
                "panel_id": "australia-atc",
            },
        },
    ]

    try:
        timezones = Timezones.objects.filter(is_enabled=True).order_by("panel_name")
    except Timezones.DoesNotExist:
        timezones = None

    if not timezones:
        timezones = AA_TIMEZONE_DEFAULT_PANELS

    context = {"title": __title__, "timezones": timezones, "timestamp": timestamp}

    return render(request, "timezones/index.html", context)
